import unittest
import tempfile
import os

from staroid import Staroid

def integration_test_ready():
    return "STAROID_ACCESS_TOKEN" in os.environ and "STAROID_ACCOUNT" in os.environ

class TestCluster(unittest.TestCase):
    @unittest.skipUnless(integration_test_ready(), "Integration test environment is not configured")
    def test_initialize(self):
        s = Staroid(access_token=os.environ["STAROID_ACCESS_TOKEN"], account=os.environ["STAROID_ACCOUNT"])
        s.get_all_accounts()

    @unittest.skipUnless(integration_test_ready(), "Integration test environment is not configured")
    def test_crud(self):
        # given
        s = Staroid(access_token=os.environ["STAROID_ACCESS_TOKEN"], account=os.environ["STAROID_ACCOUNT"])
        all_accounts = s.get_all_accounts()
        clusters_before = s.cluster().get_all()

        # when create
        c = s.cluster().create("staroid-python it-test")

        # then
        self.assertEqual("staroid-python it-test", c.name())

        # when already exists on create
        c = s.cluster().create("staroid-python it-test")

        # then
        self.assertEqual("staroid-python it-test", c.name())

        # when list
        clusters_after = s.cluster().get_all()

        # then
        self.assertEqual(len(clusters_after), len(clusters_before) + 1)

        # when delete
        s.cluster().delete("staroid-python it-test")

        # then
        clusters_after_deleted = s.cluster().get_all()
        self.assertEqual(len(clusters_after_deleted), len(clusters_before))




import requests
import logging
import json

CLUSTER_ID_MAP={
    "gcp us-west1": 1,
    "aws us-west2": 2
}

class SKE:
    def __init__(self, json):
        self.__json = json

    def name(self):
        return self.__json["name"]

    def cloud(self):
        return self.__json["cloud"]

    def region(self):
        return self.__json["region"]

class Cluster:
    def __init__(self, json):
        self.__json = json

    def id(self):
        return int(self.__json["id"])

    def name(self):
        return self.__json["name"]

    def ske(self):
        return SKE(self.__json["ske"])

    def org_id(self):
        return int(self.__json["orgId"])
    
    def type(self):
        return self.__json["type"]


class ClusterApi:
    """Cluster api"""

    def __init__(self, staroid):
        self.__staroid = staroid

    def get(self, name):
        clusters = self.get_all()
        cluster_found = None
        for c in clusters:
            if c.name() == name:
                cluster_found = c
                break

        return cluster_found

    def get_all(self):
        r = self.__staroid._api_get("orgs/{}/vc".format(self.__staroid.get_account()))
        if r.status_code == 200:
            json_object_list = json.loads(r.text)
            cluster_list = []
            for js in json_object_list:
                cluster_list.append(Cluster(js))

            return cluster_list
        else:
            logging.error("Can not get clusters {}", r.status_code)
            return None

    def create(self, name, cluster="gcp us-west1"):
        r = self.__staroid._api_post(
            "orgs/{}/vc".format(self.__staroid.get_account()),
            {
                "name": name,
                "clusterId": CLUSTER_ID_MAP[cluster]
            }
        )

        if r.status_code == 200:
            json_object = json.loads(r.text)
            return Cluster(json_object)
        elif r.status_code == 409: # already exists
            return self.get(name)
        else:
            logging.error("Can not create clusters {}", r.status_code)
            return None


    def delete(self, name):
        cluster_to_del = self.get(name)
        if cluster_to_del != None:
            r = self.__staroid._api_delete(
                "orgs/{}/vc/{}".format(self.__staroid.get_account(), cluster_to_del.id())
            )
        else:
            return None

        if r.status_code == 200:
            return cluster_to_del
        else:
            logging.error("Can not create clusters {}", r.status_code)
            return None



import requests
import logging
import json
from .commit import Commit

class Namespace:
    def __init__(self, json):
        self.__json = json

    def id(self):
        return int(self.__json["id"])

    def namespace(self):
        return self.__json["name"]

    def alias(self):
        return self.__json["instanceName"]

    def type(self):
        return self.__json["type"]
    
    def phase(self):
        return self.__json["phase"]

    def status(self):
        return self.__json["status"]

    def access(self):
        return self.__json["access"]

    def url(self):
        return self.__json["url"]

class NamespaceApi:
    """Namespace api"""

    def __init__(self, staroid, cluster):
        self.__staroid = staroid
        self.__cluster = cluster

    def get_all(self):
        r = self.__staroid._api_get(
            "orgs/{}/vc/{}/instance".format(
                self.__staroid.get_account(),
                self.__cluster.id()
            )
        )
        if r.status_code == 200:
            json_object_list = json.loads(r.text)
            namespace_list = []
            for js in json_object_list:
                namespace_list.append(Namespace(js))
            return namespace_list
        else:
            logging.error("Can not get clusters {}", r.status_code)
            return None

    def get(self, instance_name):
        namespaces = self.get_all()
        if namespaces == None:
            return None

        for ns in namespaces:
            if ns.alias() == instance_name:
                return ns

        return None

    def get_by_id(self, instance_id):
        r = self.__staroid._api_get(
            "orgs/{}/vc/{}/instance/{}".format(
                self.__staroid.get_account(),
                self.__cluster.id(),
                instance_id
            )
        )
        if r.status_code == 200:
            js = json.loads(r.text)
            return Namespace(js)
        else:
            logging.error("Can not get namespace {}", r.status_code)
            return None

    def create(self, instance_name, commit_url):
        c = Commit(commit_url)

        r = self.__staroid._api_post(
            "orgs/{}/vc/{}/instance".format(
                self.__staroid.get_account(),
                self.__cluster.id()
            ),
            {
                "provider": c.provider(),
                "owner": c.owner(),
                "repo": c.repo(),
                "branch": c.branch(),
                "commit": c.commit(),
                "instanceName": instance_name
            }
        )
        if r.status_code == 200:
            js = json.loads(r.text)
            return Namespace(js)
        elif r.status_code == 409: # already exists
            return self.get(instance_name)
        else:
            logging.error("Can not create namespace {}", r.status_code)
            return None

    def delete(self, instance_name):
        ns = self.get(instance_name)
        if ns == None:
            return None

        r = self.__staroid._api_delete(
            "orgs/{}/vc/{}/instance/{}".format(
                self.__staroid.get_account(),
                self.__cluster.id(),
                ns.id()
            )
        )
        if r.status_code == 200:
            js = json.loads(r.text)
            return Namespace(js)
        else:
            logging.error("Can not delete namespace {}", r.status_code)
            return None


    def start(self, instance_name):
        ns = self.get(instance_name)
        if ns == None:
            return None

        c = Commit(commit_url)

        r = self.__staroid._api_put(
            "orgs/{}/vc/{}/instance/{}/resume".format(
                self.__staroid.get_account(),
                self.__cluster.id(),
                ns.id()
            )
        )
        if r.status_code == 200:
            js = json.loads(r.text)
            return Namespace(js)
        else:
            logging.error("Can not start namespace {}", r.status_code)
            return None


    def stop(self, instance_name):
        ns = self.get(instance_name)
        if ns == None:
            return None

        c = Commit(commit_url)

        r = self.__staroid._api_put(
            "orgs/{}/vc/{}/instance/{}/pause".format(
                self.__staroid.get_account(),
                self.__cluster.id(),
                ns.id()
            )
        )
        if r.status_code == 200:
            js = json.loads(r.text)
            return Namespace(js)
        else:
            logging.error("Can not stop namespace {}", r.status_code)
            return None


    def shell_start(self, instance_name):
        ns = self.get(instance_name)
        if ns == None:
            return None

        c = Commit(commit_url)

        r = self.__staroid._api_post(
            "orgs/{}/vc/{}/instance/{}/shell".format(
                self.__staroid.get_account(),
                self.__cluster.id(),
                ns.id()
            )
        )
        if r.status_code == 200:
            js = json.loads(r.text)
            return Namespace(js)
        else:
            logging.error("Can not start shell {}", r.status_code)
            return None

    def shell_stop(self, instance_name):
        ns = self.get(instance_name)
        if ns == None:
            return None

        c = Commit(commit_url)

        r = self.__staroid._api_delete(
            "orgs/{}/vc/{}/instance/{}/shell".format(
                self.__staroid.get_account(),
                self.__cluster.id(),
                ns.id()
            )
        )
        if r.status_code == 200:
            js = json.loads(r.text)
            return Namespace(js)
        else:
            logging.error("Can not start shell {}", r.status_code)
            return None



import unittest
import tempfile
import os

from staroid import Staroid

def integration_test_ready():
    return "STAROID_ACCESS_TOKEN" in os.environ and "STAROID_ACCOUNT" in os.environ

class TestStaroid(unittest.TestCase):
    def test_initialize(self):
        s = Staroid()

    def test_read_config(self):
        # unset env
        at = None
        ac = None
        if "STAROID_ACCESS_TOKEN" in os.environ:
            at = os.environ["STAROID_ACCESS_TOKEN"]  
            del os.environ["STAROID_ACCESS_TOKEN"]  
        if "STAROID_ACCOUNT" in os.environ:
            ac = os.environ["STAROID_ACCOUNT"]
            del os.environ["STAROID_ACCOUNT"]

        # given
        fp = tempfile.NamedTemporaryFile()
        fp.write(b"access_token: abc\naccount: GITHUB/user1")
        fp.flush()

        # when
        s = Staroid(config_path=fp.name)

        # then
        self.assertEqual("abc", s.get_access_token())
        self.assertEqual("GITHUB/user1", s.get_account())

        # restore env
        if at != None:
            os.environ["STAROID_ACCESS_TOKEN"] = at
        if ac != None:
            os.environ["STAROID_ACCOUNT"] = ac

    @unittest.skipUnless(integration_test_ready(), "Integration test environment is not configured")
    def test_read_default_account(self):
        # given access_token is set but account is not set
        ac = None
        if "STAROID_ACCOUNT" in os.environ:
            ac = os.environ["STAROID_ACCOUNT"]
            del os.environ["STAROID_ACCOUNT"]

        # when
        s = Staroid()

        # then
        self.assertNotEqual(None, s.get_account())

        # restore env
        if ac != None:
            os.environ["STAROID_ACCOUNT"] = ac



import unittest
import tempfile
import os, time

from staroid import Staroid

def integration_test_ready():
    return "STAROID_ACCESS_TOKEN" in os.environ and "STAROID_ACCOUNT" in os.environ

def wait_for_phase(ns_api, ns, phase):
    max_wait = 300
    while ns.phase() != phase:
        time.sleep(1)
        ns = ns_api.get_by_id(ns.id())
        max_wait -= 1
        if max_wait == 0:
            break

class TestCluster(unittest.TestCase):
    @unittest.skipUnless(integration_test_ready(), "Integration test environment is not configured")
    def test_crud_namespace(self):
        # given
        s = Staroid(access_token=os.environ["STAROID_ACCESS_TOKEN"], account=os.environ["STAROID_ACCOUNT"])
        c = s.cluster().create("staroid-python it-test-namespace")

        # when create a namespace
        ns_api = s.namespace(c)
        ns = ns_api.create("instance1", "GITHUB/staroids/namespace:master")

        # then namespace becomes RUNNING
        wait_for_phase(ns_api, ns, "RUNNING")
        self.assertEqual("RUNNING", ns_api.get_by_id(ns.id()).phase())

        # when stop a namespace
        ns = ns_api.delete("instance1")

        # then namespace becomes REMOVED
        wait_for_phase(ns_api, ns, "REMOVED")
        self.assertEqual("REMOVED", ns_api.get_by_id(ns.id()).phase())

        # when delete
        s.cluster().delete("staroid-python it-test-namespace")



import os
import yaml
import logging
import requests
import json
from .cluster import ClusterApi
from .namespace import NamespaceApi

class Org:
    def __init__(self, json):
        self.__json = json

    def name(self):
        return self.__json["name"]

    def provider(self):
        return self.__json["provider"]

    def id(self):
        return int(self.__json["id"])

class User:
    def __init__(self, json):
        self.__json = json

    def name(self):
        return self.__json["name"]

    def provider(self):
        return self.__json["provider"]

    def principal(self):
        return self.__json["principal"]


class Staroid:
    """Staroid client object"""

    def __init__(self, access_token=None, account=None, config_path="~/.staroid/config.yaml"):
        self.__api_addr = "https://staroid.com/api"
        self.__access_token = None
        self.__account = None

        # 1. set from configs
        self.__read_config(config_path)

        # 2. set from env
        if "STAROID_ACCESS_TOKEN" in os.environ:
            self.__access_token = os.environ["STAROID_ACCESS_TOKEN"]

        if "STAROID_ACCOUNT" in os.environ:
            self.__account = os.environ["STAROID_ACCOUNT"]

        # 3. set from args
        if access_token != None:
            self.__access_token = access_token
        
        if account != None:
            self.__account = account

        # if account is not set, set default user account
        if self.__account == None:
            user = self.get_user()
            if user != None:
                self.__account = "{}/{}".format(user.provider(), user.principal())

    def __read_config(self, config_path):
        try:
            with open(config_path, "r") as f:
                logging.info("Read configuration from " + config_path)
                data = yaml.load(f, Loader=yaml.FullLoader)
                self.__access_token = data.get("access_token", None)
                self.__account = data.get("account", None)
        except EnvironmentError:
            pass

    def cluster(self):
        c = ClusterApi(self)
        return c

    def namespace(self, cluster):
        n = NamespaceApi(self, cluster)
        return n

    def get_access_token(self):
        return self.__access_token

    def get_account(self):
        return self.__account

    def with_account(self, account):
        self.__account = account
        return self

    def get_user(self):
        "read user information"
        if self.__access_token == None:
            return None

        r = self._api_get("auth/user")
        if r.status_code == 200:
            js = json.loads(r.text)
            return User(js)
        else:
            return None

    def get_all_accounts(self):
        r = self._api_get("orgs/")
        if r.status_code == 200:
            org_object_list = json.loads(r.text)
            org_list = []
            for js in org_object_list:
                org_list.append(Org(js))

            return org_list
        else:
            logging.error("Can't get accounts")
            return None

    def __get_request_url(self, path):
        request_url = "{}/{}".format(self.__api_addr, path)
        return request_url

    def __get_request_headers(self):
        headers = {
            "Authorization": "token {}".format(self.__access_token),
            "Content-Type": "application/json"
        }
        return headers

    def _api_get(self, path):
        url = self.__get_request_url(path);
        headers = self.__get_request_headers()

        r = requests.get(url, headers=headers)
        return r

    def _api_post(self, path, payload):
        url = self.__get_request_url(path);
        headers = self.__get_request_headers()

        r = requests.post(url, headers=headers, data=json.dumps(payload))
        return r

    def _api_put(self, path, payload):
        url = self.__get_request_url(path);
        headers = self.__get_request_headers()

        r = requests.put(url, headers=headers, data=json.dumps(payload))
        return r

    def _api_delete(self, path):
        url = self.__get_request_url(path);
        headers = self.__get_request_headers()

        r = requests.delete(url, headers=headers)
        return r



import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="staroid", # Replace with your own username
    version="0.0.3",
    license='MIT',
    author="Staroid",
    author_email="support@staroid.com",
    description="Python client library for Staroid cloud platform",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/staroids/staroid-python",
    packages=setuptools.find_packages(),
    # See https://pypi.python.org/pypi?%3Aaction=list_classifiers
    classifiers=[
        # How mature is this project? Common values are
        #   3 - Alpha
        #   4 - Beta
        #   5 - Production/Stable
        # Indicate who your project is intended for
        'Development Status :: 5 - Production/Stable',

        # Indicate who your project is intended for
        'Intended Audience :: Developers',
        'Topic :: Software Development :: Libraries',


        # Specify the Python versions you support here. In particular, ensure
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',

        # Pick your license as you wish (should match "license" above)
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    python_requires='>=3.6',
    install_requires=[
        'requests',
        'pyyaml',

    ]
)



class Commit:
    def __init__(self, commit_url):
        self.__commit_url = commit_url

        commit_hash_pos = commit_url.find("#")
        if commit_hash_pos > 0:
            self.__commit = commit_url[commit_hash_pos + 1:]
        else:
            self.__commit = ""
        
        branch_pos = commit_url.find(":")
        if branch_pos < 0:
            raise Exception("Branch is missing")

        if commit_hash_pos > 0:
            self.__branch = commit_url[branch_pos + 1:commit_hash_pos]
        else:
            self.__branch = commit_url[branch_pos + 1:]

        if len(self.__branch) == 0:
            raise Exception("Branch is missing")
            
        proj = commit_url[:branch_pos].split("/")
        if len(proj) != 3:
            raise Exception("Invalid project format")

        self.__provider = proj[0]
        self.__owner = proj[1]
        self.__repo = proj[2]

    def url(self):
        return self.__commit_url

    def provider(self):
        return self.__provider

    def owner(self):
        return self.__owner

    def repo(self):
        return self.__repo
    
    def branch(self):
        return self.__branch

    def commit(self):
        return self.__commit



import unittest
import tempfile

from staroid import Commit

class TestCommit(unittest.TestCase):
    def test_commit(self):
        test_data = [
            ["GITHUB/staroid/app:master#commit1", ["GITHUB", "staroid", "app", "master", "commit1", "false"]],
	        ["GITHUB/staroid/app:master", ["GITHUB", "staroid", "app", "master", "", "false"]],
	        ["GITHUB/staroid/app", ["", "", "", "", "", "true"]],
	        ["GITHUB/staroid/app:", ["", "", "", "", "", "true"]],
	        ["GITHUB/staroid:master", ["", "", "", "", "", "true"]]
        ]

        for data in test_data:
            
            try:
                c = Commit(data[0])
                self.assertEqual(data[1][0], c.provider())
                self.assertEqual(data[1][1], c.owner())
                self.assertEqual(data[1][2], c.repo())
                self.assertEqual(data[1][3], c.branch())
                self.assertEqual(data[1][4], c.commit())
            except:
                self.assertEqual(data[1][5], "true")
